VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CRandom"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************
' Pedigree for the Random and Seed algorithms
'
' PMMMLCG - Prime Modulus M Multiplicative Linear
' Congruential Generator
' Modified version of the random number generator proposed
' by Park & Miller in "Random Number Generators: Good
' Ones Are Hard to Find
' CACM October 1988, Vol 31, No. 10
'  - Modifications proposed by Park to provide better
'    statistical properties (i.e. more "random" - less
'    correlation between sets of generated numbers
'  - generator is of the form
'    x = ( x * A) % M
'  - Choice of A & M can radically modify the properties of
'    the generator the current values were chosen after
'    followup work to the original paper mentioned above.
'  - The generator has a period of 2^31 - 1 with numbers
'    generated in the range of 0 < x < M
'  - The generator can run on any machine with a 32-bit
'    integer, without overflow.
'  - This generator is currently running on Sun 3/50, Sparc,
'    IBM PC/XT, IBM RS/6000 just to name a few...
'****************************************************

Option Explicit

' For Random algorithm
Private Const a     As Long = 48271
Private Const M     As Long = 2147483647
Private Const MAX   As Long = 2147483645
Private Const Q     As Long = (M / a)
Private Const R     As Long = (M Mod a)
Private Const rT    As Single = 1# / MAX
'----------------------------------------------------

Private iLast       As Long
'----------------------------------------------------

Private Sub Class_Initialize()
    Call Seed(0)
End Sub
'----------------------------------------------------

'----------------------------------------------------
' Random As Long
' returns a random value in the chosen range
' - IN iMin As Long: lower bound of the range
' - IN iMax As Long: upper bound of the range
' returns the random number
'----------------------------------------------------
Public Function Random( _
  Optional ByVal iMin As Long = 0, _
  Optional ByVal iMax As Long = MAX _
) As Long
    Dim iLo As Long, _
        iHi As Long, _
        iT  As Long
    
    If iLast = 0 Then _
        Call Class_Initialize
    
    ' Can't have range larger than 2147483645
    If Abs(iMax - iMin) > MAX Then _
        Exit Function
    
    iHi = iLast / Q
    iLo = iLast Mod Q
    
    iT = (a * iLo) - (R * iHi)
    If iT >= 0 Then _
        iLast = iT _
    Else _
        iLast = iT + M
    
    ' Range is 1-2147483646; adjust range to 0-2147483645
    Random = iLast - 1
    
    If iMin <> 0 Or iMax <> MAX Then
        If iMin < iMax Then _
            Random = iMin + ((iLast - 1) Mod (iMax - iMin + 1)) _
        Else _
            Random = iMax + ((iLast - 1) Mod (iMin - iMax + 1))
    End If
End Function

'----------------------------------------------------
' RandomReal As Single
' returns a random value in the range [0; 1[ with single precision
'----------------------------------------------------
Public Function RandomReal() As Single
    RandomReal = CSng(Random * rT)
End Function

'----------------------------------------------------
' Seed
' sets the seed value for the random generator
' - IN iSeed As Long: seed to set
'   if iSeed = 0 then the current system time is taken (Timer)
'   if iSeed = -1 then the seed is reset to the last set seed
'----------------------------------------------------
Public Sub Seed(Optional ByVal iSeed As Long = -1)
    Static iLastSeed As Long
    
    Select Case iSeed
        Case -1
            If iLastSeed Then _
                iLast = iLastSeed _
            Else _
                iLast = Abs(Timer)
            
            Exit Sub
        Case 0
            iLast = Abs(Timer)
        Case Else
            iLast = Abs(iSeed)
    End Select
    
    iLastSeed = iLast
End Sub

'----------------------------------------------------
' GetSeed As Long
' returns the last entered seed value
'----------------------------------------------------
Public Function GetSeed() As Long
    GetSeed = iLast
End Function
